#ifndef __BuddhaColor_H__
#define __BuddhaColor_H__

#include <d3dx9.h>

#include "BuddhaXNAPrerequisites.h"
#include "BuddhaVector3.h"
#include "BuddhaVector4.h"

namespace BuddhaXNA
{
	/// D3DCOLOR Ɠl̃J[l
	typedef DWORD PackedColor;

	/// F
	struct Color
	{
	public:
		BYTE A;
		BYTE R;
		BYTE G;
		BYTE B;

		Color()
		{
			R = G = B = A = 0;
		}

		Color(BYTE r, BYTE g, BYTE b, BYTE a = 255)
		{
			R = r;
			G = g;
			B = b;
			A = a;
		}

		Color(const Vector3 &vector)
		{
			R = (BYTE)((int)(255 * vector.X) & 0xff);
			G = (BYTE)((int)(255 * vector.Y) & 0xff);
			B = (BYTE)((int)(255 * vector.Z) & 0xff);
			A = 255;
		}

		Color(const Vector4 &vector)
		{
			R = (BYTE)((int)(255 * vector.X) & 0xff);
			G = (BYTE)((int)(255 * vector.Y) & 0xff);
			B = (BYTE)((int)(255 * vector.Z) & 0xff);
			A = (BYTE)((int)(255 * vector.W) & 0xff);
		}

		PackedColor _getPackedColor(void) const
		{
			return D3DCOLOR_ARGB(A, R, G, B);
		}

		/**
		 * @deprecated
		 */
		static Color CreateFromFloat(float r, float g, float b, float a = 1.0f)
		{
			return Color(Vector4(r, g, b, a));
		}

		static D3DXCOLOR CreateD3DXColor(const Color &color);

		static const Color White;
		static const Color Black;
	};
}


#endif