#ifndef __BuddhaSharedPtr_H__
#define __BuddhaSharedPtr_H__

#include "BuddhaXNAPrerequisites.h"

namespace BuddhaXNA
{
	/**
	\page sharedptr_page 틤LiVFA[hj|C^

		SharedPtr BuddhaXNA ̃wb_Œ`ASTLReiɊi[\ȎQƃJEgt̓ʂ
		|C^łB
	 
		̃|C^́A^C~Õ~XɂăAvP[V̎sɏdȉeyڂ
		ɑ΂Ē`AϋɓIɗpĂ܂B܂Ãev[gNXpāAVF
		A[h|C^쐬邱Ƃł܂B
	 
	\section shared-section1 RxV
		\li eNXɕێ鑼NX̒ʏ̃|C^́AꂪbsOΏۂłȂꍇA
	        fXgN^ŉsȂB
	    \li BuddhaXNA 񋟂鋤L|C^̌^ IWiNX + Ptr łBƂ Model
	        NX̋L|C^ ModelPtr ɂȂB
	    \li L|C^ƒʏ̃|C^̗̎擾o֐񋟂ꍇAL|C^
	        GetXXXPtr()Aʏ̃|C^ GetXXX Ŏ擾B
	  
   \section shared-section2 Ǝ̃VFA[hE|C^^̒`
		SharedPtr<T>gāAÕNX̃VFA[hE|C^^pӂ邱Ƃł܂B

	 *	\code
	 *		class YourclassPtr : public SharedPtr<Yourclass>
	 *		{
	 *		public:
	 *			YourclassPtr() : SharedPtr<Yourclass>() {}
	 *			explicit YourclassPtr(Yourclass* rep) : SharedPtr<Yourclass>(rep) {}
	 *			YourclassPtr(const YourclassPtr &r) : SharedPtr<Yourclass>(r) {}
	 *
	 *		protected:
	 *			void destory(void)
	 *			{
	 *				// Ǝ̃fXgNV΁Aɏ
	 *				SharedPtr<Yourclass>::destory();
	 *			}
	 *		}
	 *	\endcode
	 */

	/**
	 * @brief QƐJEgsL|C^BÖق̃fXgN^ĂяoKvȃIuWFNgpB
	 * @see \ref sharedptr_page
	 */
	template<class T>
	class SharedPtr
	{
	protected:
		/// |C^{
		T* m_pRep;

		/// QƃJEgۑĂԒnւ̃|C^
		unsigned int* m_pUseCount;

	public:
		SharedPtr() : m_pRep(NULL), m_pUseCount(NULL)
		{
		}

		template<class Y>
		explicit SharedPtr(Y* rep)
			: m_pRep(rep),
			  m_pUseCount(new unsigned int(1))
		{
		}

		SharedPtr(const SharedPtr&r)
			: m_pRep(NULL),
			  m_pUseCount(NULL)
		{
			m_pRep = r.m_pRep;
			m_pUseCount = r.m_pUseCount;

			if (m_pUseCount != NULL)
			{
				++(*m_pUseCount);
			}
		}

		SharedPtr& operator = (const SharedPtr& r) {
			if (m_pRep == r.m_pRep)
				return *this;
			release();

			m_pRep = r.m_pRep;
			m_pUseCount = r.m_pUseCount;
			if (m_pUseCount != NULL)
			{
				++(*m_pUseCount);
			}

			return *this;
		}

		template<class Y>
		SharedPtr(const SharedPtr<Y>& r)
			: m_pRep(NULL),
			  m_pUseCount(NULL)
		{
			m_pRep = r.GetPointer();
			m_pUseCount = r.GetUseCountPointer();
			if (m_pUseCount != NULL)
			{
				++(*m_pUseCount);
			}
		}

		template<class Y>
		SharedPtr& operator = (const SharedPtr<Y>& r) {
			if (m_pRep == r.m_pRep)
				return *this;
			release();

			m_pRep = r.GetPointer();
			m_pUseCount = r.GetUseCountPointer();
			if (m_pUseCount != NULL)
			{
				++(*m_pUseCount);
			}

			return *this;
		}

		virtual ~SharedPtr()
		{
			release();
		}

		inline T& operator*() const { return *m_pRep; }
		inline T* operator->() const { return m_pRep; }
		inline T* Get() const { return m_pRep; }

		void Bind(T* rep)
		{
			m_pUseCount = new unsigned int(1);
			m_pRep = rep;
		}

		inline bool IsUnique() const { return *m_pUseCount == 1; }
		inline unsigned int GetUseCount() const { return *m_pUseCount; }
		inline unsigned int* GetUseCountPointer() const { return m_pUseCount; }

		inline T* GetPointer() const { return m_pRep; }

		inline bool IsNull(void) const { return m_pRep == NULL; }

		inline void SetNull(void)
		{
			if (m_pRep) {
				release();
				m_pRep = NULL;
				m_pUseCount = NULL;
			}
		}

	protected:
		inline void release(void)
		{
			bool isDestroyThis = false;

			if (m_pUseCount != NULL)
			{
				if (--(*m_pUseCount) == 0) {
					isDestroyThis = true;
				}
			}

			if (isDestroyThis)
				destroy();
		}

		virtual void destroy(void)
		{
			SAFE_DELETE(m_pRep);
			SAFE_DELETE(m_pUseCount);
		}
	};

	template<class T, class U> inline bool operator == (SharedPtr<T> const& a, SharedPtr<U> const& b)
	{
		return a.Get() == b.Get();
	}

	template<class T, class U> inline bool operator != (SharedPtr<T> const& a, SharedPtr<U> const& b)
	{
		return a.Get() != b.Get();
	}
}

#endif