#include "BuddhaBoundingFrustum.h"

namespace BuddhaXNA
{
	BoundingFrustum::BoundingFrustum(void)
	{
	}
	//------------------------------------------------------------------------
	BoundingFrustum::BoundingFrustum(const Matrix &value)
	{
		Left.Normal.X = value.M41 + value.M11;
		Left.Normal.Y = value.M42 + value.M12;
		Left.Normal.Z = value.M43 + value.M13;
		Left.D        = value.M44 + value.M14;
		_normalizePlane(Left);

		Right.Normal.X = value.M41 - value.M11;
		Right.Normal.Y = value.M42 - value.M12;
		Right.Normal.Z = value.M43 - value.M13;
		Right.D        = value.M44 - value.M14;
		_normalizePlane(Right);

		Top.Normal.X = value.M41 - value.M21;
		Top.Normal.Y = value.M42 - value.M22;
		Top.Normal.Z = value.M43 - value.M23;
		Top.D        = value.M44 - value.M24;
		_normalizePlane(Top);

		Bottom.Normal.X = value.M41 + value.M21;
		Bottom.Normal.Y = value.M42 + value.M22;
		Bottom.Normal.Z = value.M43 + value.M23;
		Bottom.D        = value.M44 + value.M24;
		_normalizePlane(Bottom);

		Near.Normal.X = value.M41 + value.M31;
		Near.Normal.Y = value.M42 + value.M32;
		Near.Normal.Z = value.M43 + value.M33;
		Near.D        = value.M44 + value.M34;
		_normalizePlane(Near);

		Far.Normal.X = value.M41 - value.M31;
		Far.Normal.Y = value.M42 - value.M32;
		Far.Normal.Z = value.M43 - value.M33;
		Far.D        = value.M44 - value.M34;
		_normalizePlane(Far);
	}
	//------------------------------------------------------------------------
	bool BoundingFrustum::Contains(const Vector3 &position) const
	{
		if (Near.GetSide(position) == PlaneSide::Negative)
			return false;

		if (Far.GetSide(position) == PlaneSide::Negative)
			return false;

		if (Left.GetSide(position) == PlaneSide::Negative)
			return false;

		if (Right.GetSide(position) == PlaneSide::Negative)
			return false;

		if (Top.GetSide(position) == PlaneSide::Negative)
			return false;

		if (Bottom.GetSide(position) == PlaneSide::Negative)
			return false;

		return true;
	}
}