#include "BuddhaGame.h"
#include "BuddhaGraphicsDeviceManager.h"
#include "BuddhaKeyboard.h"
#include <list>

#define	BUDDHA_DEFAULT_WINDOW_CLASS_NAME	L"BuddhaGameWindowClass"

bool g_Buddha_IsActiveWindow = false;

LRESULT WINAPI Buddha_MsgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	bool original, next;
	switch (msg)
	{
	case WM_ACTIVATE:
		original = BuddhaXNA::Game::_getInstance()->IsActive();
		next = LOWORD(wParam) != 0;
		if (original == false && next)
			BuddhaXNA::Game::_getInstance()->_isActive(LOWORD(wParam) != 0);
		else
			BuddhaXNA::Game::_getInstance()->_isActive(LOWORD(wParam) != 0);

//		BuddhaXNA::Game::_getInstance()->_isActive(LOWORD(wParam) != 0);
		break;

	case WM_SETFOCUS:
		BuddhaXNA::Game::_getInstance()->_isActive(true);
		break;

	case WM_KILLFOCUS:
		BuddhaXNA::Game::_getInstance()->_isActive(false);
		break;

	case WM_DESTROY:
		BuddhaXNA::Game::_getInstance()->Cleanup();
		PostQuitMessage(0);
		return 0;

	case WM_PAINT:
		BuddhaXNA::Game::_getInstance()->_Draw();
		ValidateRect(hWnd, NULL);
		return 0;
	}

	return DefWindowProc(hWnd, msg, wParam, lParam);
}

namespace BuddhaXNA
{
	Game* Game::_mInstance = NULL;
	//------------------------------------------------------------------------
	Game::Game(void)
		: m_pGraphicDeviceManagerInterface(NULL),
		  mIsActive(true)
	{
		m_pServices = new GameServiceContainer();
		m_pComponents = new GameComponentCollection();
		m_pGameWindow = new GameWindow();

		mGameStartTime = mLastUpdateTime = timeGetTime();
		mIsStoreLastTime = false;

		_mInstance = this;

		mIsFixedTimeStep = true;
		mTargetElapsedTime = 16;
	}
	//------------------------------------------------------------------------
	Game::~Game()
	{
		delete m_pServices;
		delete m_pComponents;
		delete m_pGameWindow;
		Keyboard::_shutdown();
	}
	//------------------------------------------------------------------------
	void Game::initializeForWindows(void)
	{
		WNDCLASSEX wc = {
			sizeof(WNDCLASSEX), CS_CLASSDC, Buddha_MsgProc, 0L, 0L,
			GetModuleHandle(NULL), NULL, NULL, NULL, NULL,
			BUDDHA_DEFAULT_WINDOW_CLASS_NAME, NULL
		};

		RegisterClassEx(&wc);

		DWORD windowStyle = WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX;

		if (m_pGraphicDeviceManagerInterface != NULL && m_pGraphicDeviceManagerInterface->GetIsFullScreen())
			windowStyle |= WS_POPUP;
		else
			windowStyle |= WS_OVERLAPPED;

		m_pGameWindow->mHandle = CreateWindow(
				wc.lpszClassName,
				m_pGameWindow->GetTitle().c_str(),
				windowStyle,
				m_pGameWindow->GetClientBounds().X,
				m_pGameWindow->GetClientBounds().Y,
				m_pGameWindow->GetClientBounds().Width,
				m_pGameWindow->GetClientBounds().Height,
				NULL, NULL, wc.hInstance, NULL
			);
	}
	//------------------------------------------------------------------------
	HRESULT Game::initD3D()
	{
		m_pGraphicDeviceManagerInterface = static_cast<GraphicsDeviceManager*>(GetServices()->GetService(String(L"GraphicsDeviceManager")));

		initializeForWindows();

		// ŗO

		m_pGraphicDeviceManagerInterface->CreateDevice();

		return S_OK;
	}
	//------------------------------------------------------------------------
	HRESULT Game::initInput()
	{
		// Create Direct Input
		return DirectInput8Create(GetModuleHandle(NULL), DIRECTINPUT_VERSION, IID_IDirectInput8, (void **)&m_pInput, NULL);
	}
	//------------------------------------------------------------------------
	void Game::Run()
	{
		if (SUCCEEDED(initD3D()) && SUCCEEDED(initInput()))
		{
			Initialize();
			m_pComponents->_initializeAll();

			LoadGraphicContent(true);

			ShowWindow(m_pGameWindow->GetHandle(), SW_SHOWDEFAULT);
			UpdateWindow(m_pGameWindow->GetHandle());

			// bZ[W[v
			MSG msg;

			PeekMessage(&msg, NULL, 0U, 0U, PM_NOREMOVE);

			while (msg.message != WM_QUIT)
			{
				if (PeekMessage(&msg, NULL, 0U, 0U, PM_REMOVE))
				{
					TranslateMessage(&msg);
					DispatchMessage(&msg);
				}
				else
				{
					if (Game::_getInstance()->IsActive())
					{
						// foCX`FbN
						HRESULT hr = m_pGraphicDeviceManagerInterface->GetGraphicsDevice()->_getD3D9Device()->TestCooperativeLevel();
						if (FAILED(hr))
						{
							if (hr == D3DERR_DEVICELOST)
							{
								::Sleep(50);
								continue;	// foCX͕s\Ԃɂ
							}

							if (hr != D3DERR_DEVICENOTRESET)
								break;	// G[Ȃ̂ŃuCN

							m_pComponents->_unloadGraphicContent(false);
							UnloadGraphicContent(false);

							// ݂(...ƂŏC FIXME)
							D3DPRESENT_PARAMETERS d3dpp = m_pGraphicDeviceManagerInterface->GetGraphicsDevice()->GetPresentationParameters();
							hr = m_pGraphicDeviceManagerInterface->GetGraphicsDevice()->_getD3D9Device()->Reset(&d3dpp);
							if (FAILED(hr))
							{
								if (hr == D3DERR_DEVICELOST)
									continue;

								if (hr == D3DERR_DRIVERINTERNALERROR)
									break;

								if (hr == D3DERR_INVALIDCALL)
									break;

								if (hr == D3DERR_OUTOFVIDEOMEMORY)
									break;

								if (hr == E_OUTOFMEMORY)
									break;

								break;
							}

							// [h
							m_pComponents->_loadGraphicContent(false);
							LoadGraphicContent(false);
						}


						Game::_getInstance()->_Update();
						Game::_getInstance()->_Draw();
					}
				}
			}
		}
	}
	//------------------------------------------------------------------------
	void Game::_Update()
	{
		DWORD time = timeGetTime();

		if (!mIsStoreLastTime) {
			mIsStoreLastTime = true;
			mLastUpdateTime = time;
		}
		else {
			// Ԓ
			for (; time - mLastUpdateTime < mTargetElapsedTime; time = timeGetTime());
		}

		// x`FbN
		DWORD t_time = time - mLastUpdateTime;

		mGameTime.mIsRunningSlowly = (t_time * 2 >= mTargetElapsedTime);

		// Kvȉ񐔕 Update R[
		while (t_time >= mTargetElapsedTime) {
			mGameTime.TotalGameTime = TimeSpan(time - mGameStartTime);
			mGameTime.ElapsedGameTime = TimeSpan(mTargetElapsedTime);

	//		mGameTime.ElapsedRealTime = 

			//
			// Update BuddhaInput.
			//
			Keyboard::GetSingleton()->_update();

			Update(mGameTime);
			m_pComponents->_update(mGameTime);

			t_time -= mTargetElapsedTime;
		}

		mLastUpdateTime = time;
	}
	//------------------------------------------------------------------------
	void Game::_Draw()
	{
	//	mGameTime.ElapsedRealTime.TotalSeconds = timeGetTime() / 1000.0f;

		m_pGraphicDeviceManagerInterface->GetGraphicsDevice()->Clear(D3DCOLOR_XRGB(0, 0, 255));

		if (BeginDraw())
		{
			Draw(mGameTime);
			m_pComponents->_draw(mGameTime);


			// ̂ւ񂪃Cxg?
			try {
				EndDraw();
			}
			catch (DeviceLostException &e)
			{
				// 
				m_pComponents->_unloadGraphicContent(false);
				UnloadGraphicContent(false);

				bool finishedReset = false;
				do
				{
					try {
						m_pGraphicDeviceManagerInterface->GetGraphicsDevice()->Reset();
						finishedReset = true;
					}
					catch (DeviceLostException &e) {
						::Sleep(50);
					}
				} while(!finishedReset);

				// Ċmۏ
				m_pComponents->_loadGraphicContent(false);
				LoadGraphicContent(false);

				EndDraw();
			}
		}
	}
	//------------------------------------------------------------------------
	bool Game::BeginDraw(void)
	{
		return m_pGraphicDeviceManagerInterface->BeginDraw();
	}
	//------------------------------------------------------------------------
	void Game::EndDraw(void)
	{
		m_pGraphicDeviceManagerInterface->EndDraw();
	}
}
