#include "BuddhaGpuEffect.h"

#include "BuddhaGraphicsDevice.h"
#include "BuddhaGpuEffectTechnique.h"
#include "BuddhaStringConvert.h"
#include "BuddhaUtils.h"
#include "BuddhaMatrix.h"
#include "BuddhaTexture2D.h"

namespace BuddhaXNA
{
	//------------------------------------------------------------------------
	GpuEffect::GpuEffect(GraphicsDevice* device)
		: m_pDevice(device),
		  m_pEffect(NULL)
	{
	}
	//------------------------------------------------------------------------
	GpuEffect::~GpuEffect()
	{
		// GtFNg̉
		SAFE_RELEASE(m_pEffect);
	}
	//------------------------------------------------------------------------
	void GpuEffect::FromFile(GraphicsDevice *device, LPCWSTR filePath, GpuEffect** out)
	{
		*out = NULL;

		String truePath = Utils::FindMediaFile(String(filePath));

		if (truePath == L"")
			throw "The file is not found.";

		GpuEffect *effect = new GpuEffect(device);
		LPD3DXBUFFER error;
		
		HRESULT hr = D3DXCreateEffectFromFile(device->_getD3D9Device(), truePath.c_str(),
			NULL, NULL, 0, NULL, &effect->m_pEffect, &error);

		if (FAILED(hr))
		{
			LPCSTR errorMessage = (LPCSTR)error->GetBufferPointer();
			/*MessageBox(NULL, error ? (LPCSTR)error->GetBufferPointer() : NULL,
				"Shader Compole Error", MB_OK);*/
			delete effect;	// ToDo O

			return;
		}

		// ZbgAbv
		// ܂eNjbN̐𐔂 -> TCY -> Zbg
		unsigned int numTechnique = 0;
		for (numTechnique = 0; effect->m_pEffect->GetTechnique(numTechnique) != NULL ; numTechnique++);

		effect->mTechniqueList.resize(numTechnique);

		for (unsigned int i = 0; i < numTechnique; i++)
		{
			effect->mTechniqueList[i] = new GpuEffectTechnique(effect, effect->m_pEffect->GetTechnique(i));
		}

		*out = effect;
	}
	//------------------------------------------------------------------------
	EffectPtr GpuEffect::FromFile(GraphicsDevice *device, LPCWSTR filePath)
	{
		GpuEffect* effect = NULL;
		FromFile(device, filePath, &effect);

		return EffectPtr(effect);
	}
	//------------------------------------------------------------------------
	void GpuEffect::SetCurrentTechnique(LPCSTR name)
	{
		SetCurrentTechnique(AsciiStringConvert::ToString(name));
	}
	//------------------------------------------------------------------------
	void GpuEffect::SetCurrentTechnique(String &name)
	{
		TechniqueMap::iterator itr = mTechniqueMap.find(name);
		if (itr != mTechniqueMap.end())
		{
			setCurrentTechnique(itr->second);
			return;
		}

		// LbVɌȂΌ
		AsciiString handleName = StringConvert::ToAsciiString(name);
		D3DXHANDLE handle = m_pEffect->GetTechniqueByName(handleName.c_str());
		if (handle == NULL)
			return; // TODO Exception

		for (int i = 0, end = GetNumTechniques(); i < end; i++){
			D3DXHANDLE t_handle = static_cast<GpuEffectTechnique*>(mTechniqueList[i])->m_pHandle;
			if (handle == t_handle) {
				// LbVo^
				mTechniqueMap.insert(TechniqueMap::value_type(name, mTechniqueList[i]));
				setCurrentTechnique(mTechniqueList[i]);
				return;
			}
		}
	}
	//------------------------------------------------------------------------
	void GpuEffect::setCurrentTechnique(EffectTechnique *technique)
	{
		m_pCurrentTechnique = technique;
		m_pEffect->SetTechnique(static_cast<GpuEffectTechnique*>(technique)->m_pHandle);
	}
	//------------------------------------------------------------------------
	void GpuEffect::_setCurrentTechnique(_BuddhaHandle handle)
	{
		for (unsigned int i = 0, end = GetNumTechniques(); i < end; i++)
		{
			D3DXHANDLE t_handle = static_cast<GpuEffectTechnique*>(mTechniqueList[i])->m_pHandle;
			if (handle == t_handle)
			{
				setCurrentTechnique(mTechniqueList[i]);
				return;
			}
		}
	}
	//------------------------------------------------------------------------
	void GpuEffect::Begin()
	{
		// vO}uVF[_L
		m_pDevice->_setSoftwareVertexProcessing(true);

		unsigned int pass = 0;
		m_pEffect->Begin(&pass, D3DXFX_DONOTSAVESTATE);

		mIsBegan = true;
	}
	//------------------------------------------------------------------------
	void GpuEffect::End()
	{
		m_pEffect->End();
		mIsBegan = false;
	}
	//------------------------------------------------------------------------
	void GpuEffect::_commit(void)
	{
		if (mIsBegan)
			m_pEffect->CommitChanges();
	}
	//------------------------------------------------------------------------
	void GpuEffect::SetParameter(LPCSTR handle, const Matrix &mat)
	{
		D3DXMATRIX d3matrix;
		Matrix::CreateD3DXMatrix(mat, d3matrix);
		m_pEffect->SetMatrix(handle, &d3matrix);
	}
	//------------------------------------------------------------------------
	void GpuEffect::SetParameter(LPCSTR handle, const Matrix *mat, unsigned int count)
	{
		//D3DXMATRIX *d3matrix = new D3DXMATRIX[count];
		//
		//for (int i = 0; i < count; i++) {
		//	Matrix::CreateD3DXMatrix(mat[i], d3matrix[i]);
		//}

		// WARN!
		m_pEffect->SetMatrixArray(handle, (D3DXMATRIX*)mat, count);

//		delete[] d3matrix;
	}
	//------------------------------------------------------------------------
	void GpuEffect::SetParameter(LPCSTR handle, const D3DXVECTOR4 &vec)
	{
		m_pEffect->SetVector(handle, &vec);
	}
	//------------------------------------------------------------------------
	void GpuEffect::SetParameter(LPCSTR handle, const Vector3 &vec)
	{
		D3DXVECTOR4 d_vec(vec.X, vec.Y, vec.Z, 1.0f);
		m_pEffect->SetVector(handle, &d_vec);
	}
	//------------------------------------------------------------------------
	void GpuEffect::SetParameter(LPCSTR handle, const Vector4 &vec)
	{
		D3DXVECTOR4 d_vec(vec.X, vec.Y, vec.Z, vec.W);
		m_pEffect->SetVector(handle, &d_vec);
	}
	//------------------------------------------------------------------------
	void GpuEffect::SetParameter(LPCSTR handle, const Texture2D *tex)
	{
		m_pEffect->SetTexture(handle, tex->_getD3D9Texture());
	}
	//------------------------------------------------------------------------
	void GpuEffect::SetParameter(LPCSTR handle, const float value)
	{
		m_pEffect->SetFloat(handle, value);
	}
	//------------------------------------------------------------------------
	_BuddhaHandle GpuEffect::_getParameterByHandle(LPCSTR name)
	{
		return m_pEffect->GetParameterByName(NULL, name);
	}
	//------------------------------------------------------------------------
	_BuddhaHandle GpuEffect::_getTechniqueByName(LPCSTR name)
	{
		return m_pEffect->GetTechniqueByName(name);
	}
}
