#include "BuddhaGraphicsDevice.h"
#include "BuddhaVertexBuffer.h"
#include "BuddhaVertexDeclaration.h"
#include "BuddhaIndexBuffer.h"
#include "BuddhaTexture2D.h"
#include "BuddhaRenderTarget2D.h"
#include "BuddhaXNAException.h"

namespace BuddhaXNA
{
	//------------------------------------------------------------------------
	GraphicsDevice::GraphicsDevice(D3DDEVTYPE deviceType, HWND renderWindowHandle, DWORD creationOptions, D3DPRESENT_PARAMETERS &presentationParameters)
		: mPresentationParameters(presentationParameters),
		  m_pBackBuffer(NULL)
	{
		m_pD3D = NULL;
		m_pD3D = Direct3DCreate9(D3D_SDK_VERSION);
		if (m_pD3D == NULL)
		{
			throw "Could not create DirectX9";
		}

		mD3D9Device = NULL;

		HRESULT hr = m_pD3D->CreateDevice(
			D3DADAPTER_DEFAULT,
			deviceType,
			renderWindowHandle,
			creationOptions,
			&presentationParameters,
			&mD3D9Device
		);

		if (hr == D3DERR_INVALIDCALL)
		{
		}

		if (hr == D3DERR_NOTAVAILABLE)
		{
			throw "DeviceNotSupportedException";
		}

		if (FAILED(hr))
			throw "Could not create a device.";

		// obNobt@̎擾
		mD3D9Device->GetBackBuffer(0, 0, D3DBACKBUFFER_TYPE_MONO, &m_pBackBuffer);

		// Create View Port

		D3DVIEWPORT9 vp;
		vp.X = 0;
		vp.Y = 0;
		vp.Width = mPresentationParameters.BackBufferWidth;
		vp.Height = mPresentationParameters.BackBufferHeight;
		vp.MinZ = 0.0f;
		vp.MaxZ = 1.0f;

		if (FAILED(mD3D9Device->SetViewport(&vp)))
			throw "Colud not set view port";


		// etc
		m_pRenderState = new BuddhaXNA::RenderState(this);
		m_pVertexStreamCollection[0] = new VertexStream(0, this);
	}
	//------------------------------------------------------------------------
	GraphicsDevice::~GraphicsDevice()
	{
		SAFE_DELETE(m_pRenderState);
		SAFE_DELETE(m_pVertexStreamCollection[0]);
		SAFE_RELEASE(mD3D9Device);
		SAFE_RELEASE(m_pD3D);
	}
	//------------------------------------------------------------------------
	void GraphicsDevice::Clear(D3DCOLOR color)
	{
		if (mD3D9Device->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_STENCIL | D3DCLEAR_ZBUFFER, color, 1.0f, 0) == D3DERR_INVALIDCALL)
			mD3D9Device->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, color, 1.0f, 0);
	}
	//------------------------------------------------------------------------
	void GraphicsDevice::Clear(DWORD options, D3DCOLOR color, float depth, int stencil)
	{
		mD3D9Device->Clear(0, NULL, options, color, depth, stencil);
	}
	//------------------------------------------------------------------------
	void GraphicsDevice::SetVertexDeclaration(VertexDeclaration *declaration)
	{
		mD3D9Device->SetVertexDeclaration(declaration->m_pVertexDeclaration);
	}
	//------------------------------------------------------------------------
	void GraphicsDevice::SetIndexBuffer(const IndexBuffer* indexBuffer)
	{
		mD3D9Device->SetIndices(indexBuffer->m_pIndexBuffer);
	}
	//------------------------------------------------------------------------
	void GraphicsDevice::SetIndexBuffer(const IndexBufferPtr &indexBuffer)
	{
		mD3D9Device->SetIndices(indexBuffer.GetPointer()->m_pIndexBuffer);
	}
	//------------------------------------------------------------------------
	RenderState::RenderState(GraphicsDevice* device)
	{
		m_pDevice = device;
		SetCullMode(CullMode::CullClockwiseFace);
	}
	//------------------------------------------------------------------------
	void GraphicsDevice::SetTexture(int stage, Texture2D* tex)
	{
		mD3D9Device->SetTexture(stage, tex->_getD3D9Texture());
	}
	//------------------------------------------------------------------------
	void GraphicsDevice::SetRenderTarget(int renderTargetIndex, RenderTarget2D* renderTarget)
	{
		if (renderTarget == NULL)
		{
			// obNobt@ɖ߂
			mD3D9Device->SetRenderTarget(renderTargetIndex, m_pBackBuffer);
		}
		else
		{
			mD3D9Device->SetRenderTarget(renderTargetIndex, renderTarget->_getD3D9Surface());
		}
	}
	//------------------------------------------------------------------------
	void GraphicsDevice::Reset(D3DPRESENT_PARAMETERS &presentationParameters)
	{
		HRESULT hr = mD3D9Device->Reset(&presentationParameters);

		if (hr == D3DERR_DEVICELOST)
			throw DeviceLostException();

		if (hr == S_OK)
			mPresentationParameters = presentationParameters;


		mPresentationParameters = presentationParameters;
	}
}