#include "BuddhaXNAPrerequisites.h"
#include "BuddhaKeyboard.h"
#include "BuddhaGame.h"

namespace BuddhaXNA
{
	Keyboard* Keyboard::m_pSingleton = NULL;
	//------------------------------------------------------------------------
	KeyboardState::KeyboardState()
	{
		ZeroMemory(mKeys, sizeof(mKeys));
	}
	//------------------------------------------------------------------------
	bool KeyboardState::IsKeyDown(int key)
	{
		if (key >= 0 && key <= KEY_NUM)
			return mKeys[key] & 0x80 ? true : false;

		return false;
	}
	//------------------------------------------------------------------------
	bool KeyboardState::IsKeyUp(int key)
	{
		if (key >= 0 && key <= KEY_NUM)
			return mKeys[key] & 0x80 ? false : true;

		return false;
	}
	//------------------------------------------------------------------------
	Keyboard::Keyboard()
		: m_pInputDevice(NULL)
	{
		_initialize();
	}
	//------------------------------------------------------------------------
	bool Keyboard::_initialize(void)
	{
		HRESULT hr;

		SAFE_RELEASE(m_pInputDevice);

		// Create Keyboard device.
		hr = Game::_getInstance()->_getDirectInput8()->CreateDevice(GUID_SysKeyboard, &m_pInputDevice, NULL);
		if (FAILED(hr))
			return false;

		// Set getting data format.
		hr = m_pInputDevice->SetDataFormat(&c_dfDIKeyboard);
		if (FAILED(hr))
			return false;

		hr = m_pInputDevice->SetCooperativeLevel(Game::_getInstance()->GetWindow()->GetHandle(), DISCL_NONEXCLUSIVE | DISCL_FOREGROUND);
		if (FAILED(hr))
			return false;

		m_pInputDevice->Acquire();

		return true;
	}
	//------------------------------------------------------------------------
	Keyboard::~Keyboard()
	{
		SAFE_RELEASE(m_pInputDevice);
	}
	//------------------------------------------------------------------------
	Keyboard* Keyboard::GetSingleton()
	{
		if (m_pSingleton == NULL)
			m_pSingleton = new Keyboard();

		return m_pSingleton;
	}
	//------------------------------------------------------------------------
	void Keyboard::_shutdown()
	{
		delete m_pSingleton;
		m_pSingleton = NULL;
	}
	//------------------------------------------------------------------------
	KeyboardState Keyboard::GetState(void)
	{
		return m_pSingleton->mState;
	}
	//------------------------------------------------------------------------
	void Keyboard::_update()
	{
		HRESULT hr = GetSingleton()->m_pInputDevice->GetDeviceState(KEY_NUM, mState.mKeys);
		if (SUCCEEDED(hr)) {
			m_pSingleton->m_pInputDevice->Acquire();
			m_pSingleton->m_pInputDevice->GetDeviceState(KEY_NUM, mState.mKeys);
		}
		else if (hr == DIERR_INPUTLOST) {
			m_pSingleton->m_pInputDevice->Acquire();
		}
		else if (hr == DIERR_NOTACQUIRED) {
			// foCXɎsĂ
			_initialize();
		}
	}
}



