#include "BuddhaModelMesh.h"

#include "BuddhaModel.h"
#include "BuddhaModelMeshPart.h"
#include "BuddhaVertexBuffer.h"
#include "BuddhaIndexBuffer.h"
#include "BuddhaEffectTechnique.h"
#include "BuddhaEffectPass.h"
#include "BuddhaXNAException.h"

namespace BuddhaXNA
{
	ModelMesh::ModelMesh(Model* model, DWORD materialNum)
		: m_pModel(model),
		  mMaterialNum(materialNum),
		  mNumVertexes(0)
	{
	}
	//------------------------------------------------------------------------
	ModelMesh::~ModelMesh()
	{
		// GtFNg
		mEffectList.clear();

		// p[g
		for (ModelMeshPartList::iterator itr = mModelMeshPartList.begin(), end = mModelMeshPartList.end();
			itr != end; ++itr)
		{
			delete (*itr);
		}

		// }eA
		for (ModelMeshMaterialList::iterator itr = mMaterialList.begin(), end = mMaterialList.end();
			itr != end; ++itr)
		{
			delete (*itr);
		}
	}
	//------------------------------------------------------------------------
	int ModelMesh::GetNumMeshPart(void) const
	{
		return static_cast<int>(mModelMeshPartList.size());
	}
	//------------------------------------------------------------------------
	ModelMeshPart* ModelMesh::GetModelMeshPart(int idx) const
	{
		if (idx >= GetNumMeshPart())
			throw "Missing Index"; //TODO

		return mModelMeshPartList[idx];
	}
	//------------------------------------------------------------------------
	void ModelMesh::Draw()
	{
		// Setup the graphic device for rendering.
		m_pModel->m_pDevice->SetIndexBuffer(mIndexBuffer);

		for (EffectList::iterator effect_itr = mEffectList.begin(), effect_end = mEffectList.end();
			effect_itr != effect_end; ++effect_itr)
		{
			(*effect_itr)->Begin();

			for (unsigned int pass = 0, numPass = (*effect_itr)->GetCurrentTechnique()->GetNumPasses();
				pass < numPass; pass++)
			{
				(*effect_itr)->GetCurrentTechnique()->GetPass(pass)->Begin();

				if (!(*effect_itr)->GetCurrentTechnique()->GetPass(pass)->_isGpuEffect())
				{
					m_pModel->m_pDevice->_setFVF(_fvf);
				}

				for (ModelMeshPartList::iterator part_itr = mModelMeshPartList.begin(), part_end = mModelMeshPartList.end();
					part_itr != part_end; ++part_itr)
				{
					if ((*effect_itr)->GetCurrentTechnique()->GetPass(pass)->_isGpuEffect())
					{
						m_pModel->m_pDevice->SetVertexDeclaration((*part_itr)->GetVertexDeclaration());
					}
					else
					{
						// n[hEFAGtFNgȂ}eAZbg
						m_pModel->m_pDevice->_setMaterial(mMaterialList[(*part_itr)->GetAttributeId()]);
					}

					m_pModel->m_pDevice->Vertices(0)->SetSource(mVertexBuffer, 0, (*part_itr)->GetVertexStride());
					
					m_pModel->m_pDevice->DrawIndexedPrimitive(
						PrimitiveType::TriangleList,
						0,
						(*part_itr)->GetBaseVertex(),
						(*part_itr)->GetNumVertex(),
						(*part_itr)->GetStartIndex(),
						(*part_itr)->GetPrimitiveCount()
					);
				}

				(*effect_itr)->GetCurrentTechnique()->GetPass(pass)->End();
			}

			(*effect_itr)->End();
		}
	}
	//------------------------------------------------------------------------
	int ModelMesh::GetNumMaterials(void) const
	{
		return static_cast<int>(mMaterialList.size());
	}
	//------------------------------------------------------------------------
	const ModelMeshMaterial* ModelMesh::GetMaterial(int idx) const
	{
		if (idx < GetNumMaterials())
			return mMaterialList[idx];

		// TODO O
		throw "Over Size";
	}
	//------------------------------------------------------------------------
	int ModelMesh::GetNumVertexes(void) const
	{
		return mNumVertexes;
	}
	//------------------------------------------------------------------------
	EffectPtr ModelMesh::GetEffectPtr(int idx) const
	{
		if (idx >= 0 && idx < GetNumEffects())
			return mEffectList[idx];

		throw ArgumentOutOfRangeException();
	}
	//------------------------------------------------------------------------
	Effect* ModelMesh::GetEffect(int idx) const
	{
		if (idx >= 0 && idx < GetNumEffects())
			return mEffectList[idx].GetPointer();

		throw ArgumentOutOfRangeException();
	}
}